package info.kramann.vr;


import processing.video.*;
import processing.core.*;

public class KameraPC implements iKamera
{
    Capture cam = null;
    public PImage holeKamerabild()
    {
        if(cam!=null && cam.available()==true) 
        {
            cam.read();
        }      
        return cam;
    }
    public void starteKamera(PApplet pap)
    {
        starteKamera(pap,0,640,480,30);
    }
    public void starteKamera(PApplet pap, int video_nr, int breite, int hoehe, int fps)
    {
        String[] cameras = Capture.list();

        if (cameras == null) 
        {
          PApplet.println("Failed to retrieve the list of available cameras, will try the default...");
          cam = new Capture(pap, 640, 480);
        } 
        else if (cameras.length == 0) 
        {
          PApplet.println("There are no cameras available for capture.");
          return;
        } 
        else 
        {
          PApplet.println("Available cameras:");
          PApplet.printArray(cameras);
          PApplet.println("Suche nach passender Kameraeinstellung:");
          int inx = findeKameraIndex(cameras);
          PApplet.println("Am besten passende kamera Nr.:"+inx);             
          cam = new Capture(pap, cameras[inx]);
          cam.start();
        }
        
    }
    public void stoppeKamera()
    {
        if(cam!=null)
        {
             cam.stop();
             cam.dispose();
        }
    }
    
    /**
    *   Hilfsmethoden
    */    
    public int findeIndexNachWort(String text, String wort)
    {
      int index = -1;
      if(text!=null && wort!=null && wort.length()<text.length())
      {
           for(int i=0;i<text.length()-wort.length();i++)
           {
               if(text.substring(i,i+wort.length()).equals(wort))
               {
                     return i+wort.length();
               }
           }
      }
      
      return index;
    }

    public int extrahiereNaechsteZahl(String text, int ab_stelle)
    {
      if(ab_stelle<0)
          return -1;
      for(int i=ab_stelle;i<text.length();i++)
      {
            if(i==text.length()-1 || !Character.isDigit(text.charAt(i)))
            {
                int ende = i;
                if(i==text.length()-1)
                    ende++;
                String szahl = text.substring(ab_stelle,ende);
                
                return (int)Double.parseDouble(szahl); 
            }
      }
      return -1;
    }

    /**
    *  Beispielzeile:
    *  [0] "name=/dev/video0,size=1920x1080,fps=5"
    */
    public int[] extrahiereEigenschaften(int[] eigenschaften, String zeile)
    {
     if(eigenschaften==null)
         eigenschaften = new int[4];
         
     //0. Video Nr.:
     int inx = findeIndexNachWort(zeile,"video");
     eigenschaften[0] = extrahiereNaechsteZahl(zeile, inx);
     //1. Breite des Videobildes
     inx = findeIndexNachWort(zeile,"size=");
     eigenschaften[1] = extrahiereNaechsteZahl(zeile, inx);
     //2. Höhe des Videobildes
     inx = findeIndexNachWort(zeile,"size="+eigenschaften[1]+"x");
     eigenschaften[2] = extrahiereNaechsteZahl(zeile, inx);
     //3. Bilder pro Sekunde
     inx = findeIndexNachWort(zeile,"fps=");
     eigenschaften[3] = extrahiereNaechsteZahl(zeile, inx);
     
     return eigenschaften;
    }

    public int bestimmeUebereinstimmungsgrad(int[] eigenschaften, int[] wunsch_eigenschaften)
    {
      int ue_grad = 0;
      if(eigenschaften[0]==wunsch_eigenschaften[0])
          ue_grad+=10; //video nr am höchsten werten
      for(int i=1;i<eigenschaften.length;i++)
          if(eigenschaften[i]==wunsch_eigenschaften[i])
            ue_grad++;
      return ue_grad;        
    }

    public int findeKameraindex(String[] cameras, int video_nr,int breite, int hoehe, int pps)
    {
        int index = 0;
        //                     videoNr., Breite, Höehe, fps
        int[] eigenschaften = {-1,-1,-1,-1};
        int[] wunsch_eigenschaften = {video_nr,breite,hoehe,pps};
        int ue_grad_best = -1;
        if(cameras!=null && cameras.length>1)
        {
          for(int i=0;i<cameras.length;i++)
          {
               extrahiereEigenschaften(eigenschaften,cameras[i]);
               int ue_grad_neu = bestimmeUebereinstimmungsgrad(eigenschaften,wunsch_eigenschaften);
               if(ue_grad_neu>ue_grad_best)
               {
                   ue_grad_best = ue_grad_neu;
                   index = i;
                 
                   PApplet.println("Bisher am besten passend: "+cameras[i]);
                   PApplet.println("Eigenschaften hier: "+eigenschaften[0]+" "+eigenschaften[1]+" "+eigenschaften[2]+" "+eigenschaften[3]);
               }
          }
        }
    
        return index;
      }

      public int findeKameraIndex(String[] cameras)
      {
        int video_nr = 0;
        int breite   = 640; 
        int hoehe    = 480;
        int pps      = 30;
        return findeKameraindex(cameras,video_nr,breite,hoehe,pps);
      }    
}